#!/usr/bin/env python
from setuptools import setup

setup(name='py-zabbix',
      version='0.5.3',
      description='Python modules for work with zabbix.',
      url='https://github.com/blacked/py-zabbix',
      author='Alexey Dubkov',
      author_email='alexey.dubkov@gmail.com',
      packages=['zabbix']
     )
